/*
 *  Authors: Ralf Corsepius (corsepiu@faw.uni-ulm.de) and
 *           Bernd Becker (becker@faw.uni-ulm.de)
 *
 *  COPYRIGHT (c) 1997-1998, FAW Ulm, Germany
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *  Modified for imora-44-256 BSP 
 *  by Radzislaw Galler <rgaller@et.put.poznan.pl> 
 *            
 *
 *  Modified to reflect Hitachi EDK SH7045F:
 *  John M. Mills (jmills@tga.com)
 *  TGA Technologies, Inc.
 *  100 Pinnacle Way, Suite 140
 *  Norcross, GA 30071 U.S.A.
 *  
 *
 *  This modified file may be copied and distributed in accordance
 *  the above-referenced license. It is provided for critique and
 *  developmental purposes without any warranty nor representation
 *  by the authors or by TGA Technologies.
 *
 *  COPYRIGHT (c) 1999.
 *  On-Line Applications Research Corporation (OAR).
 *  Copyright assigned to U.S. Government, 1994.
 *
 *  The license and distribution terms for this file may be
 *  found in the file LICENSE in this distribution or at
 *  http://www.OARcorp.com/rtems/license.html.
 *
 *  $Id: start.S,v 1.2 2000/06/12 14:59:51 joel Exp $
 */


#include <rtems/system.h>
#include <iosupp.h>
#include <rtems/score/sh_io.h>
#include <rtems/score/iosh7045.h>

/* functions defined in hw_init.c */
extern void bsp_hw_init (void);
extern void early_hw_init (void);
extern int  boot_card(int argc, char **argv, char **envp);

/* our stack */
long stack[1022] __attribute__((section(".stack")));

/* RAM vector table defined in cpu.c */
extern void * vectab[];

/* Symbols defined in linkcmds */
extern char end_vects;
extern char etext;
extern char data;
extern char edata;
extern char _bss_start;
extern char _bss_end;
extern char stack_end;

/* other useful symbols */
extern int main(int, char**);
extern void exit(int);
extern void start(void);

/****************************************************************
 *
 * Initial vector table located in ROM  - used for always reset start
 * sequence.
 */

typedef void (*fp) (void);
#define VECT_SECT          __attribute__ ((section (".vects")))

#define STACK ((fp)&stack_end)

#pragma interrupt
void isr_dummy()
{
    /* just return */
}


const fp rom_vectors[256] VECT_SECT = {
  start,                        /* 0  Power-on reset, Program counter (PC) */
  STACK,                        /* 1  Power-on reset, Stack pointer (SP)   */
  start,                        /* 2  Reserved */
  STACK,                        /* 3  Reserved */
  start,                        /* 4  general_error, Illegal instruction exception */
  isr_dummy,                    /* 5  Reserved */
  start,                        /* 6  general_error, Illegal slot exception */
  isr_dummy,                    /* 7  Reserved */
  isr_dummy,                    /* 8  Reserved */
  start,                        /* 9  general_error, CPU Address error  */
  start,                        /* 10 general_error, DMAC Address error */
  isr_dummy,                    /* 11 NMI_INT,            */
  isr_dummy,                    /* 12 User Break */
  /*************** Reserved *****************/
  isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, /* 13-20 */
  isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, /* 21-28 */
  isr_dummy, isr_dummy, isr_dummy, /* 29-31 */
  /*************** Trap instructions ********/
  isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, /* 32-36 */
  isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, /* 37-44 */
  isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, /* 45-52 */
  isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, isr_dummy, /* 53-60 */
  isr_dummy, isr_dummy, isr_dummy, /* 61-63 */
  /*************** IRQs 0-7 *****************/
  isr_dummy,                    /* 64 MA_IntHandler_IRQ0_INT, */
  isr_dummy,                    /* 65 MA_IntHandler_IRQ1_INT, */
  isr_dummy,                    /* 66 MA_IntHandler_IRQ2_INT, */
  isr_dummy,                    /* 67 MA_IntHandler_IRQ3_INT, */
  isr_dummy,                    /* 68 MA_IntHandler_IRQ4_INT, */
  isr_dummy,                    /* 69 MA_IntHandler_IRQ5_INT, */
  isr_dummy,                    /* 70 MA_IntHandler_IRQ6_INT, */
  isr_dummy,                    /* 71 MA_IntHandler_IRQ7_INT, */
  /*************** DMAC 0-3 *****************/
  isr_dummy,		        /* 72 DEI0_DMA */
  isr_dummy,                    /* 73 Reserved */
  isr_dummy, 		        /* 74 Reserved */
  isr_dummy,                    /* 75 Reserved */
  isr_dummy,		        /* 76 DEI1_DMA */
  isr_dummy,                    /* 77 Reserved */
  isr_dummy,		        /* 78 Reserved */
  isr_dummy,                    /* 79 Reserved */
  isr_dummy,                    /* 80 DEI2_DMA */
  isr_dummy,                    /* 81 Reserved */
  isr_dummy,                    /* 82 Reserved */
  isr_dummy,                    /* 83 Reserved */
  isr_dummy,                    /* 84 DEI3_DMA */
  isr_dummy,                    /* 85 Reserved */
  isr_dummy,                    /* 86 Reserved */
  isr_dummy,                    /* 87 Reserved */ 
  /*************** MTU 0 ********************/
  isr_dummy,                    /* 88 MTU0_TGI0a */ 
  isr_dummy,                    /* 89 MTU0_TGI0b */
  isr_dummy,                    /* 90 MTU0_TGI0c */
  isr_dummy,                    /* 91 MTU0_TGI0d */
  isr_dummy,                    /* 92 MTU0_TcI0v */
  isr_dummy,                    /* 93 Reserved */
  isr_dummy,                    /* 94 Reserved */
  isr_dummy,                    /* 95 Reserved */  
  /*************** MTU 1 ********************/
  isr_dummy,                    /* 96  MTU1_TGI1a */
  isr_dummy,                    /* 97  MTU1_TGI1b */
  isr_dummy,                    /* 98  Reserved */
  isr_dummy,                    /* 99  Reserved */  
  isr_dummy,                    /* 100 MTU1_TcI1v */
  isr_dummy,                    /* 101 MTU1_TcI1u */
  isr_dummy,                    /* 102 Reserved */  
  isr_dummy,                    /* 103 Reserved */  
  /*************** MTU 2 ********************/
  isr_dummy,                    /* 104 MTU2_TGI2a */
  isr_dummy,                    /* 105 MTU2_TGI2b */
  isr_dummy,                    /* 106 Reserved */
  isr_dummy,                    /* 107 Reserved */
  isr_dummy,                    /* 108 MTU2_TCI2V */
  isr_dummy,                    /* 109 MTU2_TCI2U */
  isr_dummy,                    /* 110 Reserved */
  isr_dummy,                    /* 111 Reserved */
  /*************** MTU 3 ********************/
  isr_dummy,                    /* 112 MTU3_TGI3a */
  isr_dummy,                    /* 113 MTU3_TGI3b */
  isr_dummy,                    /* 114 MTU3_TGI3c */  
  isr_dummy,                    /* 115 MTU3_TGI3d */  
  isr_dummy,                    /* 116 MTU3_TcI3v */
  isr_dummy,                    /* 117 Reserved */
  isr_dummy,                    /* 118 Reserved */
  isr_dummy,                    /* 119 Reserved */
  /*************** MTU 4 ********************/
  isr_dummy,                    /* 120 MTU4_TGI4a */
  isr_dummy,                    /* 121 MTU4_TGI4b */
  isr_dummy,                    /* 122 MTU4_TGI4c */  
  isr_dummy,                    /* 123 MTU4_TGI4d */  
  isr_dummy,                    /* 124 MTU4_TcI4v */
  isr_dummy,                    /* 125 Reserved */
  isr_dummy,                    /* 126 Reserved */
  isr_dummy,                    /* 127 Reserved */
  /*************** SCI 0 ********************/
  isr_dummy,                    /* 128 SCI0_ERI0 */
  isr_dummy,                    /* 129 SCI0_RXI0 */
  isr_dummy,                    /* 130 SCI0_TXI0 */
  isr_dummy,                    /* 131 SCI0_TEI0 */
  /*************** SCI 1 ********************/
  isr_dummy,                    /* 132 SCI1_ERI1 */
  isr_dummy,                    /* 133 SCI1_RXI1 */
  isr_dummy,                    /* 134 SCI1_TXI1 */
  isr_dummy,                    /* 135 SCI1_TEI1 */  
  /*************** A/D  ********************/ 
  isr_dummy,                    /* 136 A/D_ADI0 */
  isr_dummy,                    /* 137 A/D_ADI1 */
  isr_dummy,                    /* 138 Reserved */
  isr_dummy,                    /* 139 Reserved */  
  /*************** DTC  ********************/ 
  isr_dummy,                    /* 140 DTC_SWI */
  isr_dummy,                    /* 141 Reserved */
  isr_dummy,                    /* 142 Reserved */  
  isr_dummy,                    /* 143 Reserved */  
  /*************** CMT 0 & 1 ********************/
  isr_dummy,		        /* 144 CMT0_CMI0 */
  isr_dummy,                    /* 145 Reserved */
  isr_dummy,                    /* 146 Reserved */  
  isr_dummy,                    /* 147 Reserved */  
  isr_dummy,                    /* 148 CMT1_CMI1 */
  isr_dummy,                    /* 149 Reserved */
  isr_dummy,                    /* 150 Reserved */  
  isr_dummy,                    /* 151 Reserved */  
  /*************** WDT ********************/
  isr_dummy,		        /* 152 WDT_ITI */
  /*************** BSC ********************/
  isr_dummy,		        /* 153 BSC_CMI */
  isr_dummy,		        /* 154 Reserved */
  isr_dummy,                    /* 155 Reserved  */
  /*************** I/O ********************/
  isr_dummy,                    /* 156 I/O_OEI  */
};

/****************************************************************
 * 
 * Initialization routine written in C
 *
 */
void cstart(void)
{
    register const char *src;
    register char *dst;
    
    while(1)
    {
        early_hw_init();
        /*
         * Initialized variables get their start values
         */
        for(src = &etext, dst = &data; dst < &edata; src++, dst++)
            *dst = *src;
        /*
         * Zero out BSS
         */
        for(dst = &_bss_start; dst < &_bss_end; dst++)
            *dst = 0;
        /*
         * Initialize vectab - interrupt vector table located in RAM
         * (BSS space)
         */
        for(src = (char *)rom_vectors, dst = (char *)vectab; src < &end_vects; src++, dst++)
            *dst = *src;
        /*
         * TODO:
         * Update VBR value
         */
        dst = (char *)vectab; 
        asm volatile ("ldc %0, vbr": : "r" (dst)); 
        /*
         * Call the mainline
         */
        boot_card(0, 0, 0);
        /*
         * Exit
         */
        exit(0);
    }
}


/****************************************************************
 * 
 * Startup sequence calling cstart()
 */
asm("        .text                        ");
asm("        .align 2                     ");
asm("        .global _start               ");        
asm("_start:                              ");        
asm("        nop                          ");        
/* install the stack pointer */
/* asm("        mov.l	_stack_end, r15   ");         */
/* setup SR */
asm("	stc	sr,r0					");
asm("	mov	#0xF0,r1				");
asm("	or	r1,r0					");
asm("	ldc	r0,sr					");
/* set registers to known values */
asm("        mov   #0, r0                 ");        
asm("        mov   #1, r1                 ");        
asm("        mov   #2, r2                 ");        
asm("        mov   #3, r3                 ");        
asm("        mov   #4, r4                 ");        
asm("        mov   #5, r5                 ");        
asm("        mov   #6, r6                 ");        
asm("        mov   #7, r7                 ");        
asm("        mov   #8, r8                 ");        
asm("        mov   #9, r9                 ");        
asm("        mov   #10, r10               ");        
asm("        mov   #11, r11               ");        
asm("        mov   #12, r12               ");        
asm("        mov   #13, r13               ");        
asm("        mov   #14, r14               ");        
/* Initialize minimal hardware */
asm("        bra _cstart                  ");        
asm("        nop		          ");

